"""Rhelver makefile variable parsing."""
from pathlib import Path
import re
from typing import Any

RHELVER_PATHS = ('Makefile.rhelver', 'Makefile')

# rhelver variable values to search for in RHELVER_PATHS and the type to cast them as.
_RHELVER_VARS = {
    'RHEL_MAJOR': int,
    'RHEL_MINOR': int,
    'ZSTREAM': str,
    'RHEL_RELEASE': str,
    'RTBUILD': str,
    'AUTOMOTIVEBUILD': str,
    'AUTOMOTIVE_RELEASE': str,
    'AUTOMOTIVE_MAJOR': int,
    'AUTOMOTIVE_MINOR': int
}

# Ugly regex for parsing Makefile files.
_MAKEFILE_PATTERN = re.compile(r'^(?P<name>[a-zA-Z_]+) ?[?:]?= ?(?P<value>.*)', re.MULTILINE)


def makefile_vars(text: str, lookup_vars: dict[str, type] | None = None) -> dict[str, Any]:
    """Return the casted variable values from the given text."""
    lookup_vars = lookup_vars or _RHELVER_VARS
    results: dict[str, Any] = {}
    matches = _MAKEFILE_PATTERN.findall(text)

    while matches and len(results) != len(lookup_vars):
        name, value = matches.pop(0)

        if name in lookup_vars:
            results[name] = lookup_vars[name](value)

    return results


def repo_rhelver_vars(base_path: str | Path) -> dict[str, Any]:
    """Return a dict of the rhelver variable values found in the given local path."""
    base_path = Path(base_path)

    for path in [Path(base_path, file) for file in RHELVER_PATHS]:
        if not path.exists():
            continue

        if results := makefile_vars(path.read_text()):
            return results

    return {}
